@@ -7,8 +7,8 @@
 ;; Keywords: tools, accessibility
 ;; URL: https://gitlab.com/michelangelo-rodriguez/greader
 
-;; Package-Version: 20251019.1127
-;; Package-Revision: 3865fe9b9439
+;; Package-Version: 20251110.439
+;; Package-Revision: c3f28c1ca3d6
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -253,16 +253,27 @@ if set to t, when you call function `greader-read', that function sets a
   (setq greader-reading-mode nil))
 
 (define-obsolete-variable-alias 'greader-map 'greader-mode-map "2022")
+
+(defcustom greader-keymap-prefix "C-r"
+  "The prefix for `greader-mode' commands."
+  :type 'string
+  :group 'greader)
+
+(defvar greader-prefix-keymap
+  (let ((map (make-sparse-keymap)))
+    (define-key map (kbd "s")   #'greader-toggle-tired-mode)
+    (define-key map (kbd "r")   #'isearch-backward)
+    (define-key map (kbd "SPC") #'greader-read)
+    (define-key map (kbd "l")   #'greader-set-language)
+    (define-key map (kbd "t")   #'greader-toggle-timer)
+    (define-key map (kbd "f")   #'greader-get-attributes)
+    (define-key map (kbd "b")   #'greader-change-backend)
+    (define-key map (kbd "c") #'greader-compile-at-point)
+    map))
+
 (defvar greader-mode-map
   (let ((map (make-sparse-keymap)))
-    (define-key map (kbd "C-r s")   #'greader-toggle-tired-mode)
-    (define-key map (kbd "C-r r")   #'isearch-backward)
-    (define-key map (kbd "C-r SPC") #'greader-read)
-    (define-key map (kbd "C-r l")   #'greader-set-language)
-    (define-key map (kbd "C-r t")   #'greader-toggle-timer)
-    (define-key map (kbd "C-r f")   #'greader-get-attributes)
-    (define-key map (kbd "C-r b")   #'greader-change-backend)
-    (define-key map (kbd "C-r c") #'greader-compile-at-point)
+    (define-key map (kbd greader-keymap-prefix) greader-prefix-keymap)
     map))
 
 (defvar greader-reading-map
@@ -275,6 +286,9 @@ if set to t, when you call function `greader-read', that function sets a
     (define-key map (kbd "<left>")   #'greader-backward)
     (define-key map (kbd "<right>")   #'greader-forward)
     map))
+
+(eval-after-load 'greader-dict
+  '(define-key greader-prefix-keymap "d" greader-dict-prefix-map))
 (defvar greader-queue-mode)
 ;;;###autoload
 (define-minor-mode greader-mode
@@ -516,32 +530,17 @@ Optional argument EVENT ."
 (defun greader-build-args ()
   "Build the string that will be passed to the back-end."
   (greader-reset)
-  (let (args arg)
-    (setq arg
-	  (greader-call-backend 'rate))
-    (setq args (append `(,arg) args))
-    (cond ((greader-call-backend 'lang)
-	   (setq arg
-		 (greader-call-backend 'lang))
-	   (setq args (append `(,arg) args))))
-    (cond ((greader-call-backend 'punctuation)
-	   (setq arg (greader-call-backend 'punctuation))
-	   (setq args (append `(,arg) args))))
-    (setq greader-backend (greader-call-backend 'executable))
-    (cond
-     (
-      (not
-       (eq
-	(greader-call-backend 'extra)
-	'not-implemented))
-      (setq arg (greader-call-backend 'extra))
-      (setq args (append `(,arg) args))))
-    (catch 'deleted
-      (dolist (argument args)
-	(when (equal argument 'not-implemented)
-	  (setq args (delete argument args))
-	  (throw 'deleted t))))
-    (setq greader-backend (append `(,greader-backend) args))))
+  (let (args)
+    (push (greader-call-backend 'rate) args)
+    (when-let* ((lang (greader-call-backend 'lang)))
+      (push lang args))
+    (when-let* ((punc (greader-call-backend 'punctuation)))
+      (push punc args))
+    (when-let* ((extra (greader-call-backend 'extra)))
+      (unless (eq extra 'not-implemented)
+        (push extra args)))
+    (setq greader-backend (append greader-backend
+                                (delete 'not-implemented (nreverse args))))))
 
 (defun greader-reset ()
   "Reset greader."
@@ -640,7 +639,8 @@ function, point jumps at the last position you called command `greader-read'."
 	  (greader-read-asynchronous ". end"))))))
 
 (defun greader-stop ()
-  "Stops reading of document."
+  "Stop reading of document.
+If `greader-dict-mode' and/or `greader-dict-toggle-filters' are active, the dictionary of pronunciation rules will be updated after calling all the hooks."
   (interactive)
   (cond
    ((and (> greader-elapsed-time 0) greader-timer-flag)
@@ -653,7 +653,9 @@ function, point jumps at the last position you called command `greader-read'."
     (setq-local greader-stop-timer 0)))
   (greader-set-greader-keymap)
   (greader-tts-stop)
-  (run-hooks 'greader-after-stop-hook))
+  (run-hooks 'greader-after-stop-hook)
+  (when (or greader-dict-mode greader-dict-toggle-filters)
+    (greader-dict--update)))
 
 (defun greader-debug (arg)
   "Used to get some fast debugging.
@@ -1859,5 +1861,14 @@ to the next sentence, or when you stop the reading."
 
 
 
+(defun greader-set-map-prefix (keys)
+  "Set `greader-keymap-prefix' to KEYS for the current session."
+  (interactive "kSet greader prefix to: ")
+  (let ((key-str (key-description keys)))
+    (define-key greader-mode-map (kbd greader-keymap-prefix) nil)
+    (setq greader-keymap-prefix key-str)
+    (define-key greader-mode-map (kbd greader-keymap-prefix) greader-prefix-keymap)
+    (message "greader prefix set to %s for the current session." key-str)))
+
 (provide 'greader)
 ;;; greader.el ends here
